package org.ets.ibt.delivery.server.utility;

import java.io.File;
import java.io.IOException;
import java.io.RandomAccessFile;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.nio.channels.FileChannel;
import java.nio.channels.FileLock;
import java.security.SecureRandom;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import org.ets.ibt.delivery.server.utility.ServerProperties;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class InterProcessCommLock {

   private static final Logger Log = LoggerFactory.getLogger(InterProcessCommLock.class);
   public static final int LOCK_ACQUIRE_WAIT_DURATION = 1000;
   public static final int LOCK_ACQUIRE_RETRY_DURATION = 500;
   public static final String SHARED_LOCATION_DIRECTORY;
   private static File sharedLockLocation;
   private static Map mapFileLock;


   private static boolean acquireLock(String resourceName) {
      return acquireLockWithTime(resourceName, 1000L);
   }

   private static boolean acquireLockWithTime(String resourceName, long waitMS) {
      boolean acquired = false;
      Map var4 = mapFileLock;
      synchronized(mapFileLock) {
         InterProcessCommLock.FileLockObject flo = (InterProcessCommLock.FileLockObject)mapFileLock.get(resourceName);
         FileLock fileLock = null;
         if(flo == null && resourceName != null && waitMS > 0L) {
            RandomAccessFile randomAccessFile = null;

            try {
               long e = System.currentTimeMillis() + waitMS;
               File file = new File(sharedLockLocation, resourceName);
               randomAccessFile = new RandomAccessFile(file, "rw");
               FileChannel fileChannel = randomAccessFile.getChannel();

               while(System.currentTimeMillis() < e) {
                  fileLock = fileChannel.tryLock();
                  if(fileLock != null) {
                     mapFileLock.put(resourceName, new InterProcessCommLock.FileLockObject(randomAccessFile, fileLock));
                     writeInfo(randomAccessFile);
                     break;
                  }

                  Thread.sleep(500L);
               }
            } catch (Exception var17) {
               var17.printStackTrace();
               Log.error("error while acquiring the lock", var17);
            } finally {
               ;
            }
         }

         acquired = fileLock != null;
         Log.error("thread-id={}, action=\'acquire-lock\', resource={}, status={}", new Object[]{Long.valueOf(Thread.currentThread().getId()), resourceName, Boolean.valueOf(acquired)});
         return acquired;
      }
   }

   private static void releaseLock(String resourceName) {
      boolean released = false;
      Map var2 = mapFileLock;
      synchronized(mapFileLock) {
         InterProcessCommLock.FileLockObject flo = (InterProcessCommLock.FileLockObject)mapFileLock.get(resourceName);
         FileLock fileLock = null;
         if(flo != null) {
            fileLock = flo.getFileLock();
         }

         if(fileLock != null) {
            try {
               fileLock.release();
               fileLock = null;
               flo.getRandomAccessFile().close();
               mapFileLock.remove(resourceName);
               released = true;
            } catch (IOException var7) {
               var7.printStackTrace();
               Log.error("error while releasing the lock", var7);
            }
         }

         Log.error("thread-id={}, action=\'release-lock\', resource={}, status={}", new Object[]{Long.valueOf(Thread.currentThread().getId()), resourceName, Boolean.valueOf(released)});
      }
   }

   private static void releaseAllLocks() {
      Map var0 = mapFileLock;
      synchronized(mapFileLock) {
         Log.error("Release all locks.  thread-id={}", Long.valueOf(Thread.currentThread().getId()));
         Iterator var1 = mapFileLock.keySet().iterator();

         while(var1.hasNext()) {
            String key = (String)var1.next();
            releaseLock(key);
         }

      }
   }

   private static void writeInfo(RandomAccessFile file) {
      synchronized(file) {
         try {
            file.seek(file.length());
            getHostName();
            Thread.currentThread().getId().writeBytes((new SimpleDateFormat("yyyy.MM.dd.HH.mm.ss")).format(new Date()));
         } catch (IOException var4) {
            var4.printStackTrace();
            Log.error("error while writing lock info", var4);
         }

      }
   }

   public static String getHostName() {
      String hostName = "Unknown";

      try {
         InetAddress ex = InetAddress.getLocalHost();
         hostName = ex.getHostName();
      } catch (UnknownHostException var2) {
         Log.error("error while acquiring the lock", var2);
      }

      return hostName;
   }

   public static void runTestCode() {
      Object randomSleep = (new SecureRandom()).nextInt(5) * 100 + 100;
      System.out.println((String)randomSleep);
      InterProcessCommLock.WorkerThread thread1 = new InterProcessCommLock.WorkerThread("Resource1.txt", (long)((new SecureRandom()).nextInt(5) * 100 + 100));
      thread1.start();
      randomSleep = (new SecureRandom()).nextInt(5) * 100 + 100;
      System.out.println((String)randomSleep);
      InterProcessCommLock.WorkerThread thread1A = new InterProcessCommLock.WorkerThread("Resource1.txt", (long)randomSleep);
      thread1A.start();
      randomSleep = (new SecureRandom()).nextInt(5) * 100 + 100;
      System.out.println((String)randomSleep);
      InterProcessCommLock.WorkerThread thread2 = new InterProcessCommLock.WorkerThread("Resource2.txt", (long)randomSleep);
      thread2.start();
      randomSleep = (new SecureRandom()).nextInt(5) * 100 + 100;
      System.out.println((String)randomSleep);
      InterProcessCommLock.WorkerThread thread2A = new InterProcessCommLock.WorkerThread("Resource2.txt", (long)randomSleep);
      thread2A.start();
   }

   public static void main(String[] args) {
      runTestCode();
   }

   static {
      ServerProperties.getMediaDirectory();
      SHARED_LOCATION_DIRECTORY = File.separator;
      File var10002 = new File;
      SHARED_LOCATION_DIRECTORY.<init>(File.separator);
      sharedLockLocation = var10002;
      mapFileLock = new HashMap();
      Runtime.getRuntime().addShutdownHook(new Thread() {
         public void run() {
            InterProcessCommLock.releaseAllLocks();
         }
      });
   }

   public static class FileLockObject {

      private RandomAccessFile randomAccessFile;
      private FileLock fileLock;


      public FileLockObject(RandomAccessFile randomAccessFile, FileLock fileLock) {
         this.randomAccessFile = randomAccessFile;
         this.fileLock = fileLock;
      }

      public RandomAccessFile getRandomAccessFile() {
         return this.randomAccessFile;
      }

      public void setRandomAccessFile(RandomAccessFile randomAccessFile) {
         this.randomAccessFile = randomAccessFile;
      }

      public FileLock getFileLock() {
         return this.fileLock;
      }

      public void setFileLock(FileLock fileLock) {
         this.fileLock = fileLock;
      }
   }

   public static class WorkerThread extends Thread {

      private String myName;
      private final long timeSleep;


      WorkerThread(String name, long newtimeSleep) {
         this.myName = name;
         this.timeSleep = newtimeSleep;
      }

      public void run() {
         try {
            for(int e = 0; e < 10; ++e) {
               Class var2 = InterProcessCommLock.class;
               synchronized(InterProcessCommLock.class) {
                  if(InterProcessCommLock.acquireLock(this.myName)) {
                     Thread.sleep(this.timeSleep);
                     InterProcessCommLock.releaseLock(this.myName);
                  }
               }
            }
         } catch (InterruptedException var5) {
            var5.printStackTrace();
         }

      }
   }
}
